# Non-replicated Systematic Sampling Variance Estimators
# Wolter, K. M. 1984. An investigation of some estimators 
# of variance for systematic sampling. Journal of the American 
# Statistical Association, 79(388): 781-790.
#####################
# Assumes no missing data!
#################################

SystematicVar2<-function(Data,FPC =T,SamplingFrac=0)
{
#######################
## Variance estimator for mean count of a single systematic sample
## Estimator 'v2' in Wolter 1984, based on first order differences.
##  (n-1) df (?)
## Assumes systematic in one dimension (e.g., time, transect)
####################
# ARGUMENTS
##################
# Data - vector of observations, in sample sequence
# FPC - logical - use Finite Population Correction?  See Cochran (1977).
# SamplingFrac - Fraction (0< <1) of the population sampled.
#############
# Last Edit: 15 March 2004
# Author: Joel H. Reynolds, joel_reynolds@fws.gov
####################
# REFERENCES
############
# Cochran, W. G. 1977. Sampling, 3rd edition. Wiley & Sons.
#
# Wolter, K. 1984.  An Investigation of Some Estimators of Variance for
#   Systematic Sampling.  Journal of the American Statistical Association
#   79 (388), 781-790.
#
###################

# Check Data
    if (is.na(length(Data))) stop(warning("Only applicable to vector objects."))
    if (sum(is.na(Data))>0) stop(warning("No missing data allowed."))
    if (!is.numeric(Data)) stop(warning("Only applicable to numeric observations."))

n<-length(Data)
# Calculate first differences
    aj<-diff(Data)

# calculate sum of squares
    var2est<-(sum(aj^2)/(2*(length(aj))))/n

# apply FPC
    if (FPC) var2est<-(1-SamplingFrac)*var2est
    
    return(c("VarEst"=var2est,"DF"=n-1))
}
###################################################

SystematicVar3<-function(Data,FPC =T,SamplingFrac=0)
{
#######################
## Variance estimator for mean count of a single systematic sample
## Estimator 'v3' in Wolter 1984, based on cutting sample
## into n/2 strata, each of size 2. (n/2 df)?
##
## Assumes systematic in one dimension (e.g., time, transect)
####################
# ARGUMENTS
##################
# Data - vector of observations, in sample sequence
# FPC - logical - use Finite Population Correction?  See Cochran (1977).
# SamplingFrac - Fraction (0< <1) of the population sampled.
#############
# Last Edit: 15 March 2004
# Author: Joel H. Reynolds, joel_reynolds@fws.gov
####################
# REFERENCES
############
# Cochran, W. G. 1977. Sampling, 3rd edition. Wiley & Sons.
#
# Wolter, K. 1984.  An Investigation of Some Estimators of Variance for
#   Systematic Sampling.  Journal of the American Statistical Association
#   79 (388), 781-790.
#
###################

# Check Data
    if (is.na(length(Data))) stop(warning("Only applicable to vector objects."))
    if (sum(is.na(Data))>0) stop(warning("No missing data allowed."))
    if (!is.numeric(Data)) stop(warning("Only applicable to numeric observations."))

    n<-length(Data)
# Calculate variance in each strata
    if (n%%2==0) {
    aj<-diff(Data)
    # drop every other to keep 'strata'-specific components
    a2j<-aj[seq(1,n-1,2)]
    }
    else {
    # arbitrarily drop first (or last) observation to get
    # even sample size
    aj<-diff(Data[-1])
    # drop every other to keep 'strata'-specific components
    a2j<-aj[seq(1,n-2,2)]
    }
# calculate sum of squares
    var3est<-(sum(a2j^2)/n)/n

# apply FPC
    if (FPC) var3est<-(1-SamplingFrac)*var3est
    
    return(c("VarEst"=var3est,"DF"=n/2))
}
###################################################
SystematicVar4<-function(Data,FPC =T,SamplingFrac=0)
{
#######################
## Variance estimator for mean count of a single systematic sample
## Estimator 'v4' in Wolter 1984, based on second order differences
## which removes any linear trend.
##   df (?)
## Assumes systematic in one dimension (e.g., time, transect)
####################
# ARGUMENTS
##################
# Data - vector of observations, in sample sequence
# FPC - logical - use Finite Population Correction?  See Cochran (1977).
# SamplingFrac - Fraction (0< <1) of the population sampled.
#############
# Last Edit: 15 March 2004
# Author: Joel H. Reynolds, joel_reynolds@fws.gov
####################
# REFERENCES
############
# Cochran, W. G. 1977. Sampling, 3rd edition. Wiley & Sons.
#
# Wolter, K. 1984.  An Investigation of Some Estimators of Variance for
#   Systematic Sampling.  Journal of the American Statistical Association
#   79 (388), 781-790.
#
###################

# Check Data
    if (is.na(length(Data))) stop(warning("Only applicable to vector objects."))
    if (sum(is.na(Data))>0) stop(warning("No missing data allowed."))
    if (!is.numeric(Data)) stop(warning("Only applicable to numeric observations."))

n<-length(Data)
# Calculate second differences
    bj<-diff(Data,differences=2)

# calculate sum of squares
    var4est<-(sum(bj^2)/(6*(n-2)))/n

# apply FPC
    if (FPC) var4est<-(1-SamplingFrac)*var4est
    
    return(c("VarEst"=var4est,"DF"=n-2))
}
###################################################

SystematicVar5<-function(Data,FPC =T,SamplingFrac=0)
{
#######################
## Variance estimator for mean count of a single systematic sample
## Estimator 'v5' in Wolter 1984, based on higher order
## differences.
##
## Assumes systematic in one dimension (e.g., time, transect)
####################
# ARGUMENTS
##################
# Data - vector of observations, in sample sequence
# FPC - logical - use Finite Population Correction?  See Cochran (1977).
# SamplingFrac - Fraction (0< <1) of the population sampled.
#############
# Last Edit: 1 May 2006
# Author: Joel H. Reynolds, joel_reynolds@fws.gov
####################
# REFERENCES
############
# Cochran, W. G. 1977. Sampling, 3rd edition. Wiley & Sons.
#
# Wolter, K. 1984.  An Investigation of Some Estimators of Variance for
#   Systematic Sampling.  Journal of the American Statistical Association
#   79 (388), 781-790.
#
###################

# Check Data
    if (is.na(length(Data))) stop(warning("Only applicable to vector objects."))
    if (sum(is.na(Data))>0) stop(warning("No missing data allowed."))
    if (!is.numeric(Data)) stop(warning("Only applicable to numeric observations."))

    n<-length(Data)
# Calculate differences
    cj<-(1/2)*Data[c(5:n)]-Data[c(4:(n-1))]+Data[c(3:(n-2))]-Data[c(2:(n-3))]+(1/2)*Data[c(1:(n-4))]

# calculate sum of squares
    var5est<-(sum(cj^2)/(3.5*(n-4)))/n

# apply FPC
    if (FPC) var5est<-(1-SamplingFrac)*var5est
    
    return(c("VarEst"=var5est,"DF"=n-4))
}
###################################################
SystematicVar6<-function(Data,FPC =T,SamplingFrac=0)
{
#######################
## Variance estimator for mean count of a single systematic sample
## Estimator 'v6' in Wolter 1984: a higher order
## differences estimator initially suggested by Yates (1949).
##
## Assumes systematic in one dimension (e.g., time, transect)
####################
# ARGUMENTS
##################
# Data - vector of observations, in sample sequence
# FPC - logical - use Finite Population Correction?  See Cochran (1977).
# SamplingFrac - Fraction (0< <1) of the population sampled.
#############
# Last Edit: 15 March 2004
# Author: Joel H. Reynolds, joel_reynolds@fws.gov
####################
# REFERENCES
############
# Cochran, W. G. 1977. Sampling, 3rd edition. Wiley & Sons.
#
# Wolter, K. 1984.  An Investigation of Some Estimators of Variance for
#   Systematic Sampling.  Journal of the American Statistical Association
#   79 (388), 781-790.
#
# Yates, F. 1949.  Sampling Methods for Censuses and Surveys. London, Charles W. Griffin.
###################

# Check Data
    if (is.na(length(Data))) stop(warning("Only applicable to vector objects."))
    if (sum(is.na(Data))>0) stop(warning("No missing data allowed."))
    if (!is.numeric(Data)) stop(warning("Only applicable to numeric observations."))

    n<-length(Data)
# Calculate differences
    dj<-(1/2)*Data[c(9:n)]-Data[c(8:(n-1))]+Data[c(7:(n-2))]-Data[c(6:(n-3))]
        +Data[c(5:(n-4))]-Data[c(4:(n-5))]+Data[c(3:(n-6))]-Data[c(2:(n-7))]
        +(1/2)*Data[c(1:(n-8))]

# calculate sum of squares
    var5est<-(sum(dj^2)/(7.5*(n-8)))/n

# apply FPC
    if (FPC) var5est<-(1-SamplingFrac)*var5est
    
    return(c("VarEst"=var5est,"DF"=n-8))
}
###################################################
StratifiedVar<-function(Data,StrataSampleSize=4,StrataSize=24,FPC =T)
{
#######################
## Variance estimator for mean count of a single systematic sample
## Treats systematic sample as a stratified random sample, strata
## being defined as a sequence of multiple sample units.
## I.e., a 10-min/hr systematic sample may be treated as a
## stratified sample with each day having 6 4-hour strata.
## Based on Skalski et al. (1993).
######
## Assumes systematic in one dimension (e.g., time, transect)
####################
# ARGUMENTS
##################
# Data - vector of observations, in sample sequence
# StrataSampleSize - number of sample observations per strata; assumes no missing data.
# StrataSize - number of possible observations in each strata; assumes equal sized strata.
# FPC - logical - use Finite Population Correction?  See Cochran (1977).
#############
# Last Edit: 3 Nov 2004, Black Wednesday.
# Author: Joel H. Reynolds, joel_reynolds@fws.gov
####################
# REFERENCES
############
# Cochran, W. G. 1977. Sampling, 3rd edition. Wiley & Sons.
#
# Skalski, J. R., A. Hoffmann, B. H. Ransom, T. W. Steig. 1993.  
# Fixed-location hydroacoustic monitoring designs for estimating 
# fish passage using stratified random and systematic sampling.  
# Canadian Journal of Fisheries and Aquatic Sciences 50(6): 1208-1221.
###################
cat("NEED TO CHECK DF formula.\n")
# Check Data
    if (is.na(length(Data))) stop(warning("Only applicable to vector objects."))
    if (sum(is.na(Data))>0) stop(warning("No missing data allowed."))
    if (!is.numeric(Data)) stop(warning("Only applicable to numeric observations."))
    
    if (length(Data)%%StrataSampleSize!=0) stop(warning("Some strata have incomplete observations.\nNo missing data allowed."))
    
    n<-length(Data)
# Create Strata identifier
    Strata<-rep(seq(n/StrataSampleSize),rep(StrataSampleSize,n/StrataSampleSize))
# Calculate Strata variances
    StrataVar<-tapply(Data,Strata,function(x){sum((x-mean(x))^2)/(length(x)-1)})
    VarEst<-(StrataSize^2)*sum(StrataVar)/StrataSampleSize
    
# apply FPC
    if (FPC) VarEst<-(1-StrataSampleSize/StrataSize)*VarEst
    
    return(c("VarEst"=VarEst,"DF"=n-n/StrataSampleSize))
}
###################################################
